@@ -318,6 +318,35 @@ gail_widget_get_parent (AtkObject *accessible)
       if (parent_widget == NULL)
         return NULL;
 
+      /*
+       * For a widget whose parent is a GtkNoteBook, we return the
+       * accessible object corresponding the GtkNotebookPage containing
+       * the widget as the accessible parent.
+       */
+      if (GTK_IS_NOTEBOOK (parent_widget))
+        {
+          gint page_num;
+          GtkWidget *child;
+          GtkNotebook *notebook;
+
+          page_num = 0;
+          notebook = GTK_NOTEBOOK (parent_widget);
+          while (TRUE)
+            {
+              child = gtk_notebook_get_nth_page (notebook, page_num);
+              if (!child)
+                break;
+              if (child == widget)
+                {
+                  parent = gtk_widget_get_accessible (parent_widget);
+                  parent = atk_object_ref_accessible_child (parent, page_num);
+                  g_object_unref (parent);
+                  return parent;
+                }
+              page_num++;
+            }
+        }
+
       parent = gtk_widget_get_accessible (parent_widget);
     }
   return parent;
